/* Autogenerated by mlir-tblgen; don't manually edit */
#ifdef GEN_PASS_CLASSES

//===----------------------------------------------------------------------===//
// QuantConvertConst
//===----------------------------------------------------------------------===//

template <typename DerivedT>
class QuantConvertConstBase : public ::mlir::FunctionPass {
public:
  QuantConvertConstBase() : ::mlir::FunctionPass(::mlir::TypeID::get<DerivedT>()) {}
  QuantConvertConstBase(const QuantConvertConstBase &) : ::mlir::FunctionPass(::mlir::TypeID::get<DerivedT>()) {}

  /// Returns the command-line argument attached to this pass.
  ::llvm::StringRef getArgument() const override { return "quant-convert-const"; }

  /// Returns the derived pass name.
  ::llvm::StringRef getName() const override { return "QuantConvertConst"; }

  /// Support isa/dyn_cast functionality for the derived pass class.
  static bool classof(const ::mlir::Pass *pass) {
    return pass->getTypeID() == ::mlir::TypeID::get<DerivedT>();
  }

  /// A clone method to create a copy of this pass.
  std::unique_ptr<::mlir::Pass> clonePass() const override {
    return std::make_unique<DerivedT>(*static_cast<const DerivedT *>(this));
  }

  /// Return the dialect that must be loaded in the context before this pass.
  void getDependentDialects(::mlir::DialectRegistry &registry) const override {
    
  }

protected:
};

//===----------------------------------------------------------------------===//
// QuantConvertSimulatedQuant
//===----------------------------------------------------------------------===//

template <typename DerivedT>
class QuantConvertSimulatedQuantBase : public ::mlir::FunctionPass {
public:
  QuantConvertSimulatedQuantBase() : ::mlir::FunctionPass(::mlir::TypeID::get<DerivedT>()) {}
  QuantConvertSimulatedQuantBase(const QuantConvertSimulatedQuantBase &) : ::mlir::FunctionPass(::mlir::TypeID::get<DerivedT>()) {}

  /// Returns the command-line argument attached to this pass.
  ::llvm::StringRef getArgument() const override { return "quant-convert-simulated-quantization"; }

  /// Returns the derived pass name.
  ::llvm::StringRef getName() const override { return "QuantConvertSimulatedQuant"; }

  /// Support isa/dyn_cast functionality for the derived pass class.
  static bool classof(const ::mlir::Pass *pass) {
    return pass->getTypeID() == ::mlir::TypeID::get<DerivedT>();
  }

  /// A clone method to create a copy of this pass.
  std::unique_ptr<::mlir::Pass> clonePass() const override {
    return std::make_unique<DerivedT>(*static_cast<const DerivedT *>(this));
  }

  /// Return the dialect that must be loaded in the context before this pass.
  void getDependentDialects(::mlir::DialectRegistry &registry) const override {
    
  }

protected:
};
#undef GEN_PASS_CLASSES
#endif // GEN_PASS_CLASSES
#ifdef GEN_PASS_REGISTRATION

//===----------------------------------------------------------------------===//
// QuantConvertConst Registration
//===----------------------------------------------------------------------===//

inline void registerQuantConvertConstPass() {
  ::mlir::registerPass("quant-convert-const", "Converts constants followed by qbarrier to actual quantized values", []() -> std::unique_ptr<::mlir::Pass> {
    return mlir::quant::createConvertConstPass();
  });
}

//===----------------------------------------------------------------------===//
// QuantConvertSimulatedQuant Registration
//===----------------------------------------------------------------------===//

inline void registerQuantConvertSimulatedQuantPass() {
  ::mlir::registerPass("quant-convert-simulated-quantization", "Converts training-time simulated quantization ops to corresponding quantize/dequantize casts", []() -> std::unique_ptr<::mlir::Pass> {
    return mlir::quant::createConvertSimulatedQuantPass();
  });
}

//===----------------------------------------------------------------------===//
// Quant Registration
//===----------------------------------------------------------------------===//

inline void registerQuantPasses() {
  registerQuantConvertConstPass();
  registerQuantConvertSimulatedQuantPass();
}
#undef GEN_PASS_REGISTRATION
#endif // GEN_PASS_REGISTRATION
